;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                                  Musical Key Reference 2.1                                      /
;/                              Written by Donald Straney 3/30/03                                  /
;/                       Designed for the Z80/TI-83+ graphing calculator                         /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////DEFINITIONS/////////////////////////////////////////////////

_homeup equ $4558

#define TI83P

#include "ti83plus.inc"              ;Required include file

;VARS

AddrUp equ $9873           ;Pointers to next and previous key routines
AddrDown equ $9875

save_col equ $9877

.org $9d95                    ;Set where to start

;///////////////////////////////////////////START OF PROGRAM////////////////////////////////////////

Major:                              ;Display major keys
  res indicRun,(iy+indicFlags)      ;Turn off the idiotic run indicator
                                    ;Don't need to clear screen, top part is overwritten when bar is drawn,
                                    ;bottom part is cleared by call to ScreenClear in DrawMajor routine
  call DrawBar                      ;Draw title bar                 
  call DrawMajor                    ;Draw major keys in circle of fifths
  call SelectCharCMN                ;Select C Major as first key
  jr KeyLoop                        ;Check for keypresses

Minor:
                                    ;Don't need to clear screen, top part is overwritten when bar is drawn,
                                    ;bottom part is cleared by call to ScreenClear in DrawMinor routine
  call DrawBar                      ;Draw title bar
  call DrawMinor                    ;Draw minor keys in circle of fifths
  call SelectCharAmN                ;Select a minor as starting key

KeyLoop:                            ;Keypress handler
  B_CALL(_GetKey)                      ;Go into APD until key is pressed
  cp kUp                            ;Is up key?
  call z,GoToUp                     ;If so, call routine to select previous char
  cp kDown                          ;Is down key?
  call z,GoToDown                   ;If so, call routine to select next char
  cp kVarx                          ;Is x-Var key?
  jp z,GoToOtherKey                 ;If so, switch between major/minor
  cp kMode                          ;Is mode key?
  jp z,DrawPage                    ;If so, show theory info
  cp kClear                         ;Is clear key?
  jr nz, KeyLoop                        ;If none of the above, loop until valid key is pressed  

  B_CALL(_ClrScrn)                  ;If it was the clear key, clear the screen and exit
  ret

GoToOtherKey:                       ;Switch keys
  ld a,(KeyStatus)                  ;Get key status
  or a                              ;Is 0? (same as cp 0)
  jr z,Minor                        ;If so, switch to minor
  jr Major                          ;Otherwise, switch to major

DrawMinor:                          ;Draw minor circle of fifths
  ld a,1                            ;Show that is in minor mode
  call DrawInit                     ;Initialize LCD
  ld hl,MinorCircle1                ;Display circle of fifths
  call Draw_Newline
  ld hl,MinorCircle2
  call Draw_Newline
  ld hl,MinorCircle3
  call Draw_Newline
  ld hl,MinorCircle4
  call Draw_Newline
  ld hl,MinorCircle5
  call Draw_Newline
  ld hl,MinorCircle6
  call Draw_Newline
  ld hl,MinorCircle7
  B_CALL(_PutS)
  ret

DrawMajor:                          ;Draw major circle of fifths
  xor a                             ;Show that is in major mode
  call DrawInit                     ;Initialize LCD for draw
  ld hl,MajorCircle1                ;Display circle of fifths
  call Draw_Newline
  ld hl,MajorCircle2
  call Draw_Newline
  ld hl,MajorCircle3
  call Draw_Newline
  ld hl,MajorCircle4
  call Draw_Newline
  ld hl,MajorCircle5
  call Draw_Newline
  ld hl,MajorCircle6
  call Draw_Newline
  ld hl,MajorCircle7
  B_CALL(_PutS)
  xor a
  ret

;///////////////////////////////////////////ROUTINES////////////////////////////////////////////////

Draw_Newline:                       ;Draw text at (hl), then go to new line
  B_CALL(_PutS)
  B_CALL(_NewLine)
  ret

DrawInit:                           ;Screen init routines common to both draw routines
  ld (KeyStatus),a                  ;Set key status
  call DrawBar                  ;Clear screen, draw bar
  B_CALL(_homeup)                      ;Position cursor
  B_CALL(_NewLine)
  ret

DrawBar:                            ;Draws title bar
  B_CALL(_ClrLCD)
;  ld hl,_plotSScreen
;  ld de,_plotSScreen+1
;  ld bc,12*64
;  ld (hl),0
;  ldir
  ld hl,Bardata                     ;Set copy from pointer to title bar data
;  ld de,_plotSScreen
;  ld bc,12*7
;  ldir
;  B_CALL(_BufCpy)

  di
  call LCDWait
  ld a,$07
  out (16),a
  call LCDWait
  ld a,$20
  out (16),a
  call LCDWait
  ld a,$80
  ld d,a
  out (16),a
  ld b,7
DrawBarLoop:
  ld a,d
  out (16),a
  call LCDWait
  ld a,$20
  out (16),a
  call LCDWait
  push bc
  ld b,12
DrawBarLoop2:
  ld a,(hl)
  inc hl
  out (17),a
  call LCDWait
  djnz DrawBarLoop2

  pop bc
  inc d
  djnz DrawBarLoop
  
  ei

  set textInverse,(iy+textFlags)    ;Set for inverse text draw for title bar text
  ld hl,$0005
  ld (_penCol),hl
  ld hl,Title                       ;Point to title
  B_CALL(_VPutS)                       ;Draw title bar text
DrawKey:
  ld a,(KeyStatus)
  or a
  jr z,DrawMajorStr
DrawMinorStr:
  ld hl,MinorStr
  jr DrawStr
DrawMajorStr:
  ld hl,MajorStr
DrawStr:
  B_CALL(_VPutS)
  res textInverse,(iy+textFlags)    ;Set back to normal text draw
  ret

;Thanks to James Rubingh for this code

LCDWait:
;  push hl
  inc hl
  dec hl
;  pop hl
  ret

GoToUp:                             ;Jump to address of next key up
  ld hl,(AddrUp)
  jp (hl)

GoToDown:                           ;Jump to address of next key down
  ld hl,(AddrDown)
  jp (hl)

SelectChar:                     ;Routine to display box around char-b,c=x,y hl=string address
  push hl                       ;Save hl
  push bc                       ;Save bc
  ld a,(KeyStatus)
  or a
  call z,DrawMajor
  call nz,DrawMinor
;  call DrawKey
  pop bc                        ;Restore bc
;  push bc                       ;Save bc with starting cursor position
;  ld (_curRow),bc               ;Set cursor location from b,c
;  ld a,$d0                      ;Draw black box over text
;  push af
;  B_CALL(_PutC)
;  pop af
;  B_CALL(_PutMap)
;  pop bc                        ;Now return to char starting position again,
  ld (_curRow),bc
  set textInverse,(iy+textFlags);and draw inverse over black chars,
  pop hl                        ;(Restore hl)
  ld a,(hl)                     ;the original chars again!
  B_CALL(_PutC)
  inc hl
  ld a,(hl)
  B_CALL(_PutC)
  res textInverse,(iy+textFlags);Put back to normal text draw
  ld de,$0a43
  ld (_penCol),de
  dec hl                        ;Point back to beginning of string again

;===============_vputs_ml by Donald Straney==============
;
;48 bytes/cycles dependent on string length
;Inputs: hl-Pointer to zero-terminated string, $d6 is
;        new line
;        _penRow-Starting row for text
;        _penCol-Starting column for text
;
;Outputs: text drawn to screen
;

_vputs_ml:
  push hl                       ;Save starting position of string
  ld a,(_penCol)
  ld (save_col),a
_v_inc_loop:
  ld a,(hl)                     ;Look at current char in string
  cp $d6                        ;Is $d6? (new line char)
  jr z,_v_replace_zero          ;If so, go to new line
  or a                          ;Is 0? (end-of-string char)
  jr z,_v_disp_end               ;If so, end routine
  inc hl                        ;Point to next char
  jr _v_inc_loop                 ;and loop
_v_replace_zero:
  ld (hl),0                     ;Replace newline char with end-of-string char
  ex de,hl                      ;Save current position
  pop hl                        ;and restore beginning-of-line pointer
  push de
  B_CALL(_VPutS)                   ;Draw string
  pop de
  call _v_new_line              ;then go to new line
  ex de,hl                      ;Restore position in string at newline char
  ld (hl),$d6                   ;and restore the $d6
  inc hl                        ;and point to char after $d6
  push hl                       ;Save that as new beginning-of-line pointer
  jr _v_inc_loop                ;Go back to the loop
_v_disp_end:
  pop hl                        ;Remove beginning-of-line pointer from stack to prevent crash
  B_CALL(_VPutS)
  ret                     ;and draw last line

_v_new_line:                    ;Move small cursor down 1 row
  ld a,(_penRow)
  add a,6
  ld (_penRow),a
  ld a,(save_col)
  ld (_penCol),a
  ret

;///////////////////////////////////////SELECT CHARACTER ROUTINES///////////////////////////////////

SelectCharCMN:                      ;Set up registers for calling SelectChar (in this case, for C Major)
;  ld b,5                           ;Set char location on screen
;  ld c,1
  ld bc,$0501
  ld hl,CMN                         ;Point to data string
  call SelectChar                   ;Draw selection gfx-this is done first because hl is needed after this for previous key pointer
  ld hl,SelectCharFMN               ;Point to routine to select previous key (F Major)
  ld de,SelectCharGMN               ;Point to routine to select next key (G Major)
  jr SelectCharDMN-2                ;Set as pointers used by keypress handler

SelectCharGMN:
;  ld b,8
;  ld c,2
  ld bc,$0702
  ld hl,GMN
  call SelectChar
  ld hl,SelectCharCMN
  ld de,SelectCharDMN
  jr SelectCharAMN-2

SelectCharDMN:
;  ld b,10
;  ld c,3
  ld bc,$0903
  ld hl,DMN
  call SelectChar
  ld hl,SelectCharGMN
  ld de,SelectCharAMN
  jr SelectCharEMN-2

SelectCharAMN:
;  ld b,10
;  ld c,4
  ld bc,$0904
  ld hl,AMN
  call SelectChar
  ld hl,SelectCharDMN
  ld de,SelectCharEMN
  jr SelectCharBMN-2

SelectCharEMN:
;  ld b,10
;  ld c,5
  ld bc,$0905
  ld hl,EMN
  call SelectChar
  ld hl,SelectCharAMN
  ld de,SelectCharBMN
  jr SelectCharFMS-2

SelectCharBMN:
;  ld b,9
;  ld c,6
  ld bc,$0806
  ld hl,BMN
  call SelectChar
  ld hl,SelectCharEMN
  ld de,SelectCharFMS
  jr SelectCharGMF-2

SelectCharFMS:
;  ld b,7
;  ld c,7
  ld bc,$0607
  ld hl,FMS
  call SelectChar
  ld hl,SelectCharBMN
  ld de,SelectCharGMF
  jr SelectCharDMF-2

SelectCharGMF:
;  ld b,4
;  ld c,7
  ld bc,$0307
  ld hl,GMF
  call SelectChar
  ld hl,SelectCharFMS
  ld de,SelectCharDMF
  jr SelectCharAMF-2

SelectCharDMF:
;  ld b,3
;  ld c,6
  ld bc,$0106
  ld hl,DMF
  call SelectChar
  ld hl,SelectCharGMF
  ld de,SelectCharAMF
  jr SelectCharEMF-2

SelectCharAMF:
;  ld b,1
;  ld c,5
  ld bc,$0005
  ld hl,AMF
  call SelectChar
  ld hl,SelectCharDMF
  ld de,SelectCharEMF
  jr SelectCharBMF-2

SelectCharEMF:
;  ld b,1
;  ld c,4
  ld bc,$0004
  ld hl,EMF
  call SelectChar
  ld hl,SelectCharAMF
  ld de,SelectCharBMF
  jr SelectCharFMN-2

SelectCharBMF:
;  ld b,1
;  ld c,3
  ld bc,$0003
  ld hl,BMF
  call SelectChar
  ld hl,SelectCharEMF
  ld de,SelectCharFMN
  jr SelectCharAmN-2

SelectCharFMN:
;  ld b,3
;  ld c,2
  ld bc,$0202
  ld hl,FMN
  call SelectChar
  ld hl,SelectCharBMF
  ld de,SelectCharCMN
  jr SelectCharEmN-2



SelectCharAmN:
;  ld b,5
;  ld c,1
  ld bc,$0501
  ld hl,AmN
  call SelectChar
  ld hl,SelectCharDmN
  ld de,SelectCharEmN
  jr SelectCharBmN-2

SelectCharEmN:
;  ld b,8
;  ld c,2
  ld bc,$0702
  ld hl,EmN
  call SelectChar
  ld hl,SelectCharAmN
  ld de,SelectCharBmN
  jr SelectCharFmS-2

SelectCharBmN:
;  ld b,10
;  ld c,3
  ld bc,$0903
  ld hl,BmN
  call SelectChar
  ld hl,SelectCharEmN
  ld de,SelectCharFmS
  jr SelectCharCmS-2

SelectCharFmS:
;  ld b,10
;  ld c,4
  ld bc,$0904
  ld hl,FmS
  call SelectChar
  ld hl,SelectCharBmN
  ld de,SelectCharCmS
  jr SelectCharGmS-2

SelectCharCmS:
;  ld b,10
;  ld c,5
  ld bc,$0905
  ld hl,CmS
  call SelectChar
  ld hl,SelectCharFmS
  ld de,SelectCharGmS
  jr SelectCharDmS-2

SelectCharGmS:
;  ld b,9
;  ld c,6
  ld bc,$0806
  ld hl,GmS
  call SelectChar
  ld hl,SelectCharCmS
  ld de,SelectCharDmS
  jr SelectCharEmF-2

SelectCharDmS:
;  ld b,7
;  ld c,7
  ld bc,$0607
  ld hl,DmS
  call SelectChar
  ld hl,SelectCharGmS
  ld de,SelectCharEmF
  jr SelectCharBmF-2

SelectCharEmF:
;  ld b,4
;  ld c,7
  ld bc,$0307
  ld hl,EmF
  call SelectChar
  ld hl,SelectCharDmS
  ld de,SelectCharBmF
  jr SelectCharFmN-2

SelectCharBmF:
;  ld b,2
;  ld c,6
  ld bc,$0106
  ld hl,BmF
  call SelectChar
  ld hl,SelectCharEmF
  ld de,SelectCharFmN
  jr SelectCharCmN-2

SelectCharFmN:
;  ld b,1
;  ld c,5
  ld bc,$0005
  ld hl,FmN
  call SelectChar
  ld hl,SelectCharBmF
  ld de,SelectCharCmN
  jr SelectCharGmN-2

SelectCharCmN:
;  ld b,1
;  ld c,4
  ld bc,$0004
  ld hl,CmN
  call SelectChar
  ld hl,SelectCharFmN
  ld de,SelectCharGmN
  jr SetKeys

SelectCharGmN:
;  ld b,1
;  ld c,3
  ld bc,$0003
  ld hl,GmN
  call SelectChar
  ld hl,SelectCharCmN
  ld de,SelectCharDmN
  jr SetKeys

SelectCharDmN:
;  ld b,6
;  ld c,1
  ld bc,$0202
  ld hl,DmN
  call SelectChar
  ld hl,SelectCharGmN
  ld de,SelectCharAmN

SetKeys:                        ;Set hl as next key pointer, de as last key pointer
  ld (AddrUp),hl
  ex de,hl
  ld (AddrDown),hl
  ret

;///////////////////////////////////////RESOURCE STRINGS////////////////////////////////////////////

KeyStatus: .db 0                    ;Key status-0=Major, 1=Minor

MajorCircle1: .db "     C",0        ;Text for major part of circle of 5ths
MajorCircle2: .db "  F    G",0      ;Positions of keys: C=5,1 F=3,2 G=8,2
MajorCircle3: .db "Bb       D",0    ;Bb=1,3 D=10,3
MajorCircle4: .db "Eb       A",0    ;Eb=1,4 A=10,4
MajorCircle5: .db "Ab       E",0    ;Ab=1,5 E=10,5
MajorCircle6: .db " Db     B",0     ;Db=1,6 B=9,6
MajorCircle7: .db "   Gb F#",0      ;Gb=4,7 F#=7,7

MinorCircle1: .db "     a",0        ;Text for minor part of circle of 5ths
MinorCircle2: .db "  d    e",0      ;Positions of keys: a=5,1 d=3,2 e=8,2
MinorCircle3: .db "g        b",0    ;g=1,3 b=10,3
MinorCircle4: .db "c        f#",0   ;c=1,4 f#=10,4
MinorCircle5: .db "f        c#",0   ;f=1,5 c#=10,5
MinorCircle6: .db " bb     g#",0    ;bb=2,6 g#=9,6
MinorCircle7: .db "   eb d#",0      ;eb=4,7 d#=7,7

CMN: .db "C : No #",0               ;Major data strings
GMN: .db "G : F#",0
DMN: .db "D : F,C#",0
AMN: .db "A : F,C,",$d6,"G#",0
EMN: .db "E : F,C,",$d6,"G,D#",0
BMN: .db "B : F,C,",$d6,"G,D,A#",0
FMS: .db "F#: F,C,",$d6,"G,D,A,",$d6,"E#",0

FMN: .db "F : Bb",0
BMF: .db "Bb: B,Eb",0
EMF: .db "Eb: B,E,",$d6,"Ab",0
AMF: .db "Ab: B,E,",$d6,"A,Db",0
DMF: .db "Db: B,E,",$d6,"A,D,Gb",0
GMF: .db "Gb: B,E,",$d6,"A,D,G,",$d6,"Cb",0

AmN: .db "a : No #",0           ;Minor data strings
EmN: .db "e : F#",0
BmN: .db "b : F,C#",0
FmS: .db "f#: F,C,",$d6,"G#",0
CmS: .db "c#: F,C,",$d6,"G,D#",0
GmS: .db "g#: F,C,",$d6,"G,D,A#",0
DmS: .db "d#: F,C,",$d6,"G,D,A,",$d6,"E#",0

DmN: .db "d : Bb",0
GmN: .db "g : B,Eb",0
CmN: .db "c : B,E,",$d6,"Ab",0
FmN: .db "f : B,E,",$d6,"A,Db",0
BmF: .db "bb: B,E,",$d6,"A,D,Gb",0
EmF: .db "eb: F,C,",$d6,"G,D,A,",$d6,"E#",0

Title: .db "Music Reference 2.0: ",0  ;Bar title
MajorStr: .db "Major",0
MinorStr: .db "Minor",0

Bardata:
.db 00011111b                       ;Title bar data-7 lines
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111000b

.db 00111111b           
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 00111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111100b

.db 01111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 01111111b               
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b
;
.db 01111111b               
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111110b

.db 11111111b               
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b
.db 11111111b

;///////////////////////////////////////////////////////////////////////////////////////////////////
;/                      Musical Key Reference 2.1-Music Theory Subprogram                          /
;/                              Written by Donald Straney 9/21/03                                  /
;/                          Designed for the Z80/TI-83+ graphing calculator                        /
;///////////////////////////////////////////////////////////////////////////////////////////////////

;///////////////////////////////////////////START OF PROGRAM////////////////////////////////////////

DrawPage:
    call DrawBar_2                   ;Clear screen, draw title bar
    ld hl,$0800                     ;Set inital small cursor position
    ld (_penCol),hl
    ld hl,(CurrTextPos)             ;Get pointer to current position in text
    call _vputs_ml_2                  ;and draw the text
KeyLoop2:
    B_CALL(_GetKey)                 ;Now wait for keypress
    cp kDown                        ;Is it the down key?
    jr z,MoveDown
    cp kUp                          ;Is it the up key?
    jr z,MoveUp
    cp kClear                         ;Is clear key?
    jp z,Major                            ;If it is, return to major key display
    jr KeyLoop2                        ;If none of the above, loop until valid key is pressed  

MoveDown:
    ld hl,(CurrTextPos)             ;Get text pointer
MoveDownLoop:
    ld a,(hl)                       ;Get the symbol at the current position
    or a                            ;Is it 0 (end-of-string char)?
    jr z,KeyLoop2                    ;If it is, don't move down
    cp $d6                          ;Is it a newline char?
    jr z,MoveDownDone
    inc hl                          ;If not, increment pointer
    jr MoveDownLoop                 ;and loop again
MoveDownDone:                       ;If it was a newline char,
    inc hl                          ;point to the beginning of the next line
    ld (CurrTextPos),hl             ;put that into the current text position pointer
    jr DrawPage                     ;and draw the page with the updated pointer

MoveUp:
    ld hl,(CurrTextPos)             ;Get text pointer
    dec hl                          ;Decrement the pointer twice to skip the $d6 at the end of
    dec hl                          ;the previous line
MoveUpLoop:
    ld a,(hl)                       ;Get the symbol at the current position
    or a                            ;Is it 0 (end-of-string char)?
    jr z,KeyLoop2                    ;If it is, don't move up
    cp $d6                          ;Is it a newline char?
    jr z,MoveUpDone
    dec hl                          ;If not, decrement pointer
    jr MoveUpLoop                   ;and loop again
MoveUpDone:                         ;If it was a newline char,
    inc hl                          ;point to the beginning of the next line
    ld (CurrTextPos),hl             ;put that into the current text position pointer
    jr DrawPage                     ;and draw the page with the updated pointer

;///////////////////////////////////////////ROUTINES////////////////////////////////////////////////

DrawBar_2:                          ;Draws title bar
  B_CALL(_ClrLCD)
  ld hl,Bardata                     ;Set copy from pointer to title bar data
  di
  call LCDWait
  ld a,$07
  out (16),a
  call LCDWait
  ld a,$20
  out (16),a
  call LCDWait
  ld a,$80
  ld d,a
  out (16),a
  ld b,7
DrawBarLoop_2:
  ld a,d
  out (16),a
  call LCDWait
  ld a,$20
  out (16),a
  call LCDWait
  push bc
  ld b,12
DrawBarLoop2_2:
  ld a,(hl)
  inc hl
  out (17),a
  call LCDWait
  djnz DrawBarLoop2_2

  pop bc
  inc d
  djnz DrawBarLoop_2
  
  ei

  set textInverse,(iy+textFlags)    ;Set for inverse text draw for title bar text
  ld hl,$0009
  ld (_penCol),hl
  ld hl,Title_2                       ;Point to title
  B_CALL(_VPutS)                       ;Draw title bar text
  res textInverse,(iy+textFlags)
  ret

;===============_vputs_ml by Donald Straney==============
;
;54 bytes
;Inputs: hl-Pointer to zero-terminated string, $d6 is
;        new line
;        _penRow-Starting row for text
;        _penCol-Starting column for text
;
;Outputs: text drawn to screen
;

_vputs_ml_2:
  push hl                       ;Save starting position of string
_v_inc_loop_2:
  ld a,(hl)                     ;Look at current char in string
  cp $d6                        ;Is $d6? (new line char)
  jr z,_v_replace_zero_2        ;If so, go to new line
  or a                          ;Is 0? (end-of-string char)
  jp z,_v_disp_end              ;If so, end routine
  inc hl                        ;Point to next char
  jr _v_inc_loop_2              ;and loop
_v_replace_zero_2:
  ld (hl),0                     ;Replace newline char with end-of-string char
  ex de,hl                      ;Save current position
  pop hl                        ;and restore beginning-of-line pointer
  push de
  B_CALL(_VPutS)                ;Draw string
  pop de
  call _v_new_line_2            ;then go to new line
  ex de,hl                      ;Restore position in string at newline char
  ld (hl),$d6                   ;restore the $d6
  ld a,(_penRow)
  cp 62                         ;Is it past the end of the screen?
  ret nc                        ;Don't continue if it is
  inc hl                        ;Otherwise, point to char after $d6
  push hl                       ;Save that as new beginning-of-line pointer
  jr _v_inc_loop_2              ;Go back to the loop

_v_new_line_2                   ;Move small cursor down 1 row
  ld a,(_penRow)
  add a,6
  ld (_penRow),a
  xor a
  ld (_penCol),a
  ret

;///////////////////////////////////////RESOURCE STRINGS////////////////////////////////////////////

Title_2: .db "Music Theory Reference",0

CurrTextPos: .dw TextStart          ;Pointer to current position in text

.db 0,$d6
TextStart:
.db "FORMING CHORDS WITH SCALE DEGREES:",$d6
.db "- is half step down, + is half step up",$d6,$d6
.db "major-1,3,5",$d6
.db "minor-1,3-,5",$d6
.db "augmented-1,3,5+",$d6
.db "diminished-1,3-,5-",$d6
.db "suspended fourth-1,4,5",$d6
.db "suspended second-1,2,5",$d6
.db "fifth-1,5"
.db $d6
.db "sixth-1,3,5,6",$d6
.db "seventh-1,3,5,7-",$d6
.db "major seventh-1,3,5,7",$d6
.db "minor seventh-1,3-,5,7-",$d6
.db "ninth-1,3,5,7-,9",$d6
.db "major ninth-1,3,5,7,9",$d6
.db "minor ninth-1,3-,5,7-,9",$d6
.db "eleventh-1,3,5,7-,9,11",$d6
.db "minor eleventh-1,3-,5,7-,",$d6,"9,11",$d6
.db "thirteenth-1,3,5,7-,",$d6,"9,11,13",$d6,$d6
.db "FORMING SCALES WITH WHOLE,",$d6,"HALF, AND 3/2 STEPS:",$d6,$d6
.db "major-W,W,H,W,W,W,H",$d6
.db "natural minor-W,H,W,W,H,W,",$d6,"H",$d6
.db "harmonic minor-W,H,W,W,H,",$d6
.db "W,3/2,H",$d6
.db "melodic minor-W,H,W,W,W,",$d6,"W,H (up)",$d6
.db "(down is natural minor)",$d6
.db "minor pentatonic-W,3/2,W,",$d6,"W,W,3/2,W",$d6
.db "major pentatonic-W,W,W,",$d6,"3/2,W,W,3/2",$d6
.db "diminished-W,H,W,H,W,H,W,H"
.db $d6,$d6
.db "ORDER OF SHARPS:",$d6,$d6
.db "F,C,G,D,A,E,B",$d6,$d6
.db "ORDER OF FLATS:",$d6,$d6
.db "B,E,A,D,G,C,F"
.db $d6,$d6
.db "MODES:",$d6
.db "ionian-(major)",$d6
.db "dorian-(minor w/6+)",$d6
.db "phrygian-(minor w/2-)",$d6
.db "lydian-(major w/4+)",$d6
.db "mixolydian-(major w/7-)",$d6
.db "aeolian-(minor)",$d6
.db "locrian-(minor w/2-,5-)",0
.db 0

.end
